@@ -15,7 +15,7 @@ logger = logging.getLogger(__name__)
 class MMDDrift:
     def __init__(
             self,
-            x_ref: np.ndarray,
+            x_ref: Union[np.ndarray, list],
             backend: str = 'tensorflow',
             p_val: float = .05,
             preprocess_x_ref: bool = True,
@@ -93,7 +93,7 @@ class MMDDrift:
             self._detector = MMDDriftTorch(*args, **kwargs)  # type: ignore
         self.meta = self._detector.meta
 
-    def predict(self, x: np.ndarray, return_p_val: bool = True, return_distance: bool = True) \
+    def predict(self, x: Union[np.ndarray, list], return_p_val: bool = True, return_distance: bool = True) \
             -> Dict[Dict[str, str], Dict[str, Union[int, float]]]:
         """
         Predict whether a batch of data has drifted from the reference data.
