@@ -7,7 +7,7 @@ from alibi_detect.models.tensorflow.gmm import gmm_energy, gmm_params
 from alibi_detect.models.tensorflow.losses import loss_aegmm
 from alibi_detect.models.tensorflow.trainer import trainer
 from alibi_detect.base import BaseDetector, FitMixin, ThresholdMixin, outlier_prediction_dict
-from alibi_detect.utils.prediction import predict_batch
+from alibi_detect.utils.tensorflow.prediction import predict_batch
 
 logger = logging.getLogger(__name__)
 
@@ -166,7 +166,7 @@ class OutlierAEGMM(BaseDetector, FitMixin, ThresholdMixin):
         -------
         Array with outlier scores for each instance in the batch.
         """
-        _, z, _ = predict_batch(self.aegmm, X, batch_size=batch_size)
+        _, z, _ = predict_batch(X, self.aegmm, batch_size=batch_size)
         energy, _ = gmm_energy(z, self.phi, self.mu, self.cov, self.L, self.log_det_cov, return_mean=False)
         return energy.numpy()
 
